@@ -34,3 +34,10 @@
  */
 
 #undef NO_PRIVATE
+
+/*
+ * Try once more on failed DNS lookups (to allow far away connections 
+ * which might resolve slowly)
+ */
+
+#define	DNS_TRYAGAIN
