@@ -34,7 +34,11 @@
 #include <stdio.h>
 #include <errno.h>
 #include <string.h>
+#ifdef SYSSYSLOG
+#include <sys/syslog.h>
+#else
 #include <syslog.h>
+#endif
 #include <time.h>
 #include <ctype.h>
 #include <pwd.h>
@@ -175,16 +179,15 @@ hostmatch(char *addr)
          * in access file
          */
         if ((incfile = fopen(addr, "r")) == NULL) {
-            if (errno != ENOENT)
-                syslog(LOG_ERR,
-                       "cannot open addrglob file %s: %s", addr,
-                       strerror(errno));
+            if (errno != ENOENT) syslog(LOG_ERR,
+		        "cannot open addrglob file %s: %s", addr, strerror(errno));
             return(0);
         }
         
         while (!found && (fgets(incline, MAXLINE, incfile) != NULL)) {
             ptr = strtok(incline, " \t\n");
-            found = 1;
+            if (ptr && hostmatch(ptr))
+            	found = 1;
             while (!found && ((ptr = strtok(NULL, " \t\n")) != NULL)) {
                 if (ptr && hostmatch(ptr))
                     found = 1;
