@@ -1,9 +1,44 @@
+/* Copyright (c) 1993, 1994  Washington University in Saint Louis
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are
+ * met: 1. Redistributions of source code must retain the above copyright
+ * notice, this list of conditions and the following disclaimer. 2.
+ * Redistributions in binary form must reproduce the above copyright notice,
+ * this list of conditions and the following disclaimer in the documentation
+ * and/or other materials provided with the distribution. 3. All advertising
+ * materials mentioning features or use of this software must display the
+ * following acknowledgement: This product includes software developed by the
+ * Washington University in Saint Louis and its contributors. 4. Neither the
+ * name of the University nor the names of its contributors may be used to
+ * endorse or promote products derived from this software without specific
+ * prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY WASHINGTON UNIVERSITY AND CONTRIBUTORS
+ * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL WASHINGTON
+ * UNIVERSITY OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
+ * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
+ * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+ * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
+ * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
+ * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ */
+
 #include "config.h"
 
 #include <stdio.h>
 #include <errno.h>
 #include <string.h>
+#ifdef SYSSYSLOG
+#include <sys/syslog.h>
+#else
 #include <syslog.h>
+#endif
 #include <time.h>
 #include <pwd.h>
 #include <grp.h>
@@ -23,7 +58,7 @@
 #include "extensions.h"
 #include "support/ftw.h"
 
-#ifdef HAS_REGEX_H
+#ifdef HAVE_REGEX_H
 #include <regex.h>
 #endif
 
@@ -690,7 +725,7 @@ fn_check(char *name)
               return(0);
           }
 #if defined(REGEXEC)
-          if (regexec(&regexbuf, path, 1, &regmatchbuf, 0) != 0)
+          if (regexec(&regexbuf, path, 1, &regmatchbuf, 0) != 0) {
 #elif defined(REGEX)
 # ifdef M_UNIX
           regp = regex(sp, path);
@@ -749,21 +784,23 @@ fn_check(char *name)
 }
 
 int
-dir_check(char *name, uid_t *uid, gid_t *gid)
+dir_check(char *name, uid_t *uid, gid_t *gid, int *valid)
 {
   struct aclmember *entry = NULL;
 
   int i,
     match_value = -1;
-  char *ap2,
-       *ap3,
-       *ap4,
-       *ap6;
+  char *ap2 = NULL,
+       *ap3 = NULL,
+       *ap4 = NULL,
+       *ap6 = NULL;
   char cwdir[BUFSIZ];
   char path[BUFSIZ];
   char *sp;
   extern struct passwd *pw;
 
+  *valid = 0;
+
   strcpy(path, name);
   if (sp = strrchr(path, '/'))  *sp = '\0';
   else strcpy(path, ".");
@@ -788,7 +825,7 @@ dir_check(char *name, uid_t *uid, gid_t *gid)
       }
   }
 
-  if ( ((!strcasecmp(ap2, "no")) && (ap3 && strcasecmp(ap3, "dirs"))) || 
+  if ( ((ap2 && !strcasecmp(ap2, "no")) && (ap3 && strcasecmp(ap3, "dirs"))) || 
         (ap3 && !strcasecmp(ap3, "nodirs")) ||
 		(ap6 && !strcasecmp(ap6, "nodirs")) ) {
       reply(530, "%s: Permission denied.  (Upload)", name);
@@ -797,13 +834,15 @@ dir_check(char *name, uid_t *uid, gid_t *gid)
 
   if (ap3)
      *uid = atoi(ap3);    /* the uid  */
-  if (ap4)
+  if (ap4) {
      *gid = atoi(ap4);    /* the gid  */
+     *valid = 1;
+   }
   return(1);
 }
 
 int
-upl_check(char *name, int *uid, int *gid, int *f_mode)
+upl_check(char *name, uid_t *uid, gid_t *gid, int *f_mode, int *valid)
 {
   int  match_value = -1;
   char cwdir[BUFSIZ];
@@ -811,15 +850,17 @@ upl_check(char *name, int *uid, int *gid, int *f_mode)
   char *sp;
   int  i;
 
-  char *ap1,
-   *ap2,
-   *ap3,
-   *ap4,
-   *ap5;
+  char *ap1 = NULL,
+   *ap2 = NULL,
+   *ap3 = NULL,
+   *ap4 = NULL,
+   *ap5 = NULL;
 
   struct aclmember *entry = NULL;
   extern struct passwd *pw;
 
+  *valid = 0;
+
       /* what's our current directory? */
 
       strcpy(path, name);
@@ -860,8 +901,10 @@ upl_check(char *name, int *uid, int *gid, int *f_mode)
           if (!strcasecmp(ap2, "yes")) {
               if (ap3)
                   *uid = atoi(ap3);    /* the uid  */
-              if (ap4)
+              if (ap4) {
                   *gid = atoi(ap4);    /* the gid  */
+		  *valid = 1;
+		}
               if (ap5)
                   sscanf(ap5, "%o", f_mode); /* the mode */
           } else {
