@@ -72,7 +72,11 @@ static char sccsid[] = "@(#)ftpd.c  5.40 (Berkeley) 7/2/91";
 #include <errno.h>
 #include <string.h>
 #include <varargs.h>
+#ifdef SYSSYSLOG
+#include <sys/syslog.h>
+#else
 #include <syslog.h>
+#endif
 #include <time.h>
 #include "conversions.h"
 #include "extensions.h"
@@ -123,6 +127,7 @@ static char sccsid[] = "@(#)ftpd.c  5.40 (Berkeley) 7/2/91";
 /* File containing login names NOT to be used on this machine. Commonly used
  * to disallow uucp. */
 extern int errno;
+extern int pidfd;
 extern char *ctime(const time_t *);
 #ifndef NO_CRYPT_PROTO
 extern char *crypt(const char *, const char *);
@@ -290,7 +295,7 @@ main(int argc, char **argv, char **envp)
     data_source.sin_port = htons(ntohs(ctrl_addr.sin_port) - 1);
     debug = 0;
 
-#ifdef  LOG_LOCAL7
+#ifdef FACILITY
     openlog("ftpd", LOG_PID | LOG_NDELAY, FACILITY);
 #else
     openlog("ftpd", LOG_PID);
@@ -512,7 +517,11 @@ main(int argc, char **argv, char **envp)
           }
     } else {
 #ifdef SVR4
+#ifndef AIX
           strcpy(ls_long,"/bin/ls -la");
+#else
+          strcpy(ls_long,"/bin/ls -lA");
+#endif
 #else
           strcpy(ls_long,"/bin/ls -lgA");
 #endif
@@ -528,7 +537,11 @@ main(int argc, char **argv, char **envp)
       }
     } else {
 #ifdef SVR4
+#ifndef AIX
           strcpy(ls_short,"/bin/ls -la");
+#else
+          strcpy(ls_short,"/bin/ls -lA");
+#endif
 #else
           strcpy(ls_short,"/bin/ls -lgA");
 #endif
@@ -704,13 +717,112 @@ user(char *name)
     }
 
     anonymous = 0;
+    acl_remove();
 
     if (!strcasecmp(name, "ftp") || !strcasecmp(name, "anonymous")) {
-        if (checkuser("ftp") || checkuser("anonymous")) {
-            reply(530, "User %s access denied.", name);
-            syslog(LOG_NOTICE,
-                "FTP LOGIN REFUSED (ftp in /etc/ftpusers) FROM %s [%s], %s",
-                   remotehost, remoteaddr, name);
+      struct aclmember *entry = NULL;
+      int machineok=1;
+      char guestservername[MAXHOSTNAMELEN];
+      guestservername[0]='\0';
+
+      if (checkuser("ftp") || checkuser("anonymous")) {
+          reply(530, "User %s access denied.", name);
+          syslog(LOG_NOTICE,
+	       "FTP LOGIN REFUSED (ftp in /etc/ftpusers) FROM %s [%s], %s",
+	       remotehost, remoteaddr, name);
+          return;
+          
+        /*
+        ** Algorithm used:
+        ** - if no "guestserver" directive is present,
+        **     anonymous access is allowed, for backward compatibility.
+        ** - if a "guestserver" directive is present,
+        **     anonymous access is restricted to the machines listed,
+        **     usually the machine whose CNAME on the current domain
+        **     is "ftp"...
+        **
+        ** the format of the "guestserver" line is
+        ** guestserver [<machine1> [<machineN>]]
+        ** that is, "guestserver" will forbid anonymous access on all machines
+        ** while "guestserver ftp inf" will allow anonymous access on
+        ** the two machines whose CNAMES are "ftp.enst.fr" and "inf.enst.fr".
+        **
+        ** if anonymous access is denied on the current machine,
+        ** the user will be asked to use the first machine listed (if any)
+        ** on the "guestserver" line instead:
+        ** 530- Guest login not allowed on this machine,
+        **      connect to ftp.enst.fr instead.
+        **
+        ** -- <Nicolas.Pioch@enst.fr>
+        */
+      } else if (getaclentry("guestserver", &entry)
+                 && ARG0 && strlen(ARG0) > 0) {
+        struct hostent *tmphostent;
+
+        /*
+        ** if a "guestserver" line is present,
+        ** default is not to allow guest logins
+        */
+        machineok=0;
+
+        if (hostname[0]
+            && ((tmphostent=gethostbyname(hostname)))) {
+
+          /*
+          ** hostname is the only first part of the FQDN
+          ** this may or may not correspond to the h_name value
+          ** (machines with more than one IP#, CNAMEs...)
+          ** -> need to fix that, calling gethostbyname on hostname
+          **
+          ** WARNING!
+          ** for SunOS 4.x, you need to have a working resolver in the libc
+          ** for CNAMES to work properly.
+          ** If you don't, add "-lresolv" to the libraries before compiling!
+          */
+          char dns_localhost[MAXHOSTNAMELEN];
+          int machinecount;
+
+          strncpy(dns_localhost,
+                  tmphostent->h_name,
+                  sizeof(dns_localhost));
+          dns_localhost[sizeof(dns_localhost)-1]='\0';
+
+          for (machinecount=0;
+               entry->arg[machinecount] && (entry->arg[machinecount])[0];
+               machinecount++) {
+
+            if ((tmphostent=gethostbyname(entry->arg[machinecount]))) {
+              /*
+              ** remember the name of the first machine for redirection
+              */
+
+              if ((!machinecount) && tmphostent->h_name) {
+                strncpy(guestservername, entry->arg[machinecount],
+                        sizeof(guestservername));
+                guestservername[sizeof(guestservername)-1]='\0';
+              }
+
+              if (!strcasecmp(tmphostent->h_name, dns_localhost)) {
+                machineok++;
+                break;
+              }
+            }
+          }
+        }
+      }
+      if (!machineok) {
+        if (guestservername[0])
+          reply(530,
+             "Guest login not allowed on this machine, connect to %s instead.",
+                guestservername);
+        else
+          reply(530,
+                "Guest login not allowed on this machine.");
+        syslog(LOG_NOTICE,
+               "FTP LOGIN REFUSED (localhost not in guestservers) FROM %s [%s], %s",
+               remotehost, remoteaddr, name);
+        /* End of the big patch -- Nap */
+
         } else if ((pw = sgetpwnam("ftp")) != NULL) {
             anonymous = 1;      /* for the access_ok call */
             if ((why = access_ok(530)) == 1) {
@@ -758,7 +870,7 @@ user(char *name)
         /* after they log in successfully                                  */
         guest = acl_guestgroup(pw);
     }
-    if (!access_ok(530)) {
+    if (access_ok(530) < 1) {
         reply(530, "User %s access denied....", name);
         syslog(LOG_NOTICE, "FTP LOGIN REFUSED (access denied) FROM %s [%s], %s",
                remotehost, remoteaddr, name);
@@ -880,10 +992,10 @@ pass(char *passwd)
         if (*passwd == '-')
             passwd++;
 #ifdef SHADOW_PASSWORD
-        if (pw && *pw->pw_passwd == 'x' && ! *(pw->pw_passwd+1) ) {
-            struct spwd *spw = getspnam( pw->pw_name );
-            if( !spw ) { pw->pw_passwd = ""; }
-            else { pw->pw_passwd = spw->sp_pwdp; }
+        if (pw) {
+           struct spwd *spw = getspnam( pw->pw_name );
+           if( !spw ) { pw->pw_passwd = ""; }
+           else { pw->pw_passwd = spw->sp_pwdp; }
         }
 #endif
 
@@ -911,6 +1023,7 @@ pass(char *passwd)
             syslog(LOG_INFO, "failed login from %s [%s], %s",
                               remotehost, remoteaddr, the_user);
 #endif
+            acl_remove();
 
             pw = NULL;
             if (++login_attempts >= lgi_failure_threshold) {
@@ -945,6 +1058,7 @@ pass(char *passwd)
                 lreply(530, "[%s will be added if password ends with @]",
                        remotehost);
                 reply(530, "Login incorrect.");
+		acl_remove();	
                 if (++login_attempts >= lgi_failure_threshold) {
                     syslog(LOG_NOTICE, "repeated login failures from %s [%s]",
                            remotehost, remoteaddr);
@@ -1349,14 +1463,18 @@ store(char *name, char *mode, int unique)
 #endif /* OVERWRITE */
 
 #ifdef UPLOAD
+    int open_flags = (O_RDWR | O_CREAT |
+		      ((mode && *mode == 'a') ? O_APPEND : O_TRUNC));
+
     mode_t oldmask;
     int f_mode = -1,
       dir_mode,
       match_value = -1;
-    int uid = -1,
-      gid = -1;
+    uid_t uid;
+    gid_t gid;
     uid_t oldid;
     int trunc = O_TRUNC;
+    int valid = 0;
 
 #endif /* UPLOAD */
 
@@ -1364,13 +1482,21 @@ store(char *name, char *mode, int unique)
         (name = gunique(name)) == NULL)
         return;
 
+    /*
+     * check the filename, is it legal?
+     */
+    if ( (fn_check(name)) <= 0 )
+        return;
+
 #ifdef OVERWRITE
     /* if overwrite permission denied and file exists... then deny the user
      * permission to write the file. */
     while (getaclentry("overwrite", &entry) && ARG0 && ARG1 != NULL) {
         if (type_match(ARG1))
-            if (strcmp(ARG0, "yes"))
+            if (strcmp(ARG0, "yes") != 0) {
                 overwrite = 0;
+                open_flags |= O_EXCL;
+            }
     }
 
     if (!overwrite && !stat(name, &st)) {
@@ -1379,14 +1505,8 @@ store(char *name, char *mode, int unique)
     }
 #endif /* OVERWRITE */
 
-    /*
-     * check the filename, is it legal?
-     */
-    if ( (fn_check(name)) <= 0 )
-        return;
-
 #ifdef UPLOAD
-    if ( (match_value = upl_check(name, &uid, &gid, &f_mode)) < 0 )
+    if ( (match_value = upl_check(name, &uid, &gid, &f_mode, &valid)) < 0 )
         return;
 
     /* Only truncate on open if the file is not to be appended to. */
@@ -1400,10 +1520,10 @@ store(char *name, char *mode, int unique)
      * else open the file and let the default umask determine the file mode. */
     if (f_mode >= 0) {
         oldmask = umask(0000);
-        fdout = open(name, O_RDWR | O_CREAT | trunc, f_mode);
+        fdout = open(name, open_flags, f_mode);
         umask(oldmask);
     } else
-        fdout = open(name, O_RDWR | O_CREAT | trunc, 0666);
+        fdout = open(name, open_flags, 0666);
 
     if (fdout < 0) {
         perror_reply(553, name);
@@ -1411,11 +1531,14 @@ store(char *name, char *mode, int unique)
     }
     /* if we have a uid and gid, then use them. */
 
-    if (uid != (uid) -1) {
+    if (valid > 0) {
         oldid = geteuid();
         (void) seteuid((uid_t) 0);
-        if ((chown(name, uid, gid)) < 0)
-            perror_reply(550, "chown");
+        if ((fchown(fdout, uid, gid)) < 0) {
+            (void) seteuid(oldid);
+            perror_reply(550, "fchown");
+            return;
+        }
         (void) seteuid(oldid);
     }
 #endif /* UPLOAD */
@@ -1831,7 +1954,7 @@ statcmd(void)
     u_char *a,
      *p;
 
-    lreply(211, "%s FTP server status:", hostname, version);
+    lreply(211, "%s FTP server status:", hostname);
     printf("     %s\r\n", version);
     printf("     Connected to %s", remotehost);
     if (!isdigit(remotehost[0]))
@@ -2052,7 +2175,7 @@ delete(char *name)
     if ( (del_check(name)) == 0 )
         return;
 
-    if (stat(name, &st) < 0) {
+    if (lstat(name, &st) < 0) {
         perror_reply(550, name);
         return;
     }
@@ -2127,14 +2250,14 @@ cwd(char *path)
 
 makedir(char *name)
 {
-	uid_t uid = (uid_t) -1;
-	gid_t gid = (gid_t) -1;
-	uid_t oldid;
+	uid_t uid;
+	gid_t gid;
+	int   valid = 0;
 
     /*
      * check the directory, can we mkdir here?
      */
-    if ( (dir_check(name, &uid, &gid)) <= 0 )
+    if ( (dir_check(name, &uid, &gid, &valid)) <= 0 )
         return;
 
     /*
@@ -2143,28 +2266,23 @@ makedir(char *name)
     if ( (fn_check(name)) <= 0 )
         return;
 
-    if (mkdir(name, 0777) < 0)
+    if (mkdir(name, 0777) < 0) {
         perror_reply(550, name);
-    else
-        reply(257, "MKD command successful.");
-
-    if (uid != (uid) -1) {
-        oldid = geteuid();
-        (void) seteuid((uid_t) 0);
-        if ((chown(name, uid, gid)) < 0)
-            perror_reply(550, "chown");
-        (void) seteuid(oldid);
+	return;
     }
+
+    reply(257, "MKD command successful.");
 }
 
 removedir(char *name)
 {
 	int c, d;  /* dummy variables */
+        int valid = 0;
 
     /*
      * check the directory, can we rmdir here?
      */
-    if ( (dir_check(name, &c, &d)) <= 0 )
+    if ( (dir_check(name, &c, &d, &valid)) <= 0 )
         return;
 
     /*
@@ -2204,7 +2322,7 @@ renamefrom(char *name)
 {
     struct stat st;
 
-    if (stat(name, &st) < 0) {
+    if (lstat(name, &st) < 0) {
         perror_reply(550, name);
         return ((char *) 0);
     }
@@ -2300,8 +2418,10 @@ dologout(int status)
         (void) seteuid((uid_t) 0);
         logwtmp(ttyline, "", "");
     }
+    syslog(LOG_INFO, "FTP session closed");
     if (xferlog)
         close(xferlog);
+    acl_remove();
     /* beware of flushing buffers after a SIGPIPE */
     _exit(status);
 }
